/*
 * File:   Main.c
 * Author: sandepka
 *
 * Created on April 13, 2015, 9:17 AM
 */
#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

#include "stdio.h"
#include "stdlib.h"
#include "math.h"
#include "Macro_defination.h"

// The controller runs at 1 Mhz internal frequency
// CONFIG
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select bit (MCLR pin function is MCLR)
#pragma config CP = OFF         // Code Protection bit (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)
#pragma config BOREN = OFF      // Brown Out Detect (BOR disabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is disabled)


/*
 *
 */

unsigned int ADCBUF[4];
unsigned int *pointADC;
float PV_Energy;
long switch_status;
PARAMETER parameter;
CONTROL function;
ENERGY pmtr;
CONDITION pv,grid,press_sw;
signed char grid_check=0;
signed int PV_presence=0;
signed char initiate_EEPROMwrite=0;
extern void initialisation_EEPROM(void);
extern void initialisation_IOpins(void);
extern void configure_oscillator(void);
extern void initialisation_ANALOG(void);
extern void init_timer0(void);
extern void Average_Parameters(unsigned int *);
extern void write_EEPROM(unsigned char ,unsigned char );
extern unsigned char read_EEPROM(unsigned char address);
/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/
unsigned int i=0;
//unsigned char grid;
float PV_power,BAT_power,PV_Energy,BAT_Energy,Actual_Energy_remains;

void main(void)
{
    unsigned int AnalogData;
    static unsigned int pv_secondcal=0,batt_secondcal=0;
    //static unsigned int on_condition_status=0;
    //static unsigned int off_condition_status=0;
    
    /* Configure the oscillator for the device */
    //ConfigureOscillator();

    /* Initialize I/O and Peripherals for application */
    //InitApp();
    initialisation_EEPROM();
    initialisation_IOpins();
    configure_oscillator();
    initialisation_ANALOG();
    /* Read EEPROM value*/
    pmtr.PV_Energy.SPLIT[1]=read_EEPROM(0x00);  // Higher byte read from 0x00
    pmtr.PV_Energy.SPLIT[0]=read_EEPROM(0x01);  // Lower byte read from 0x01
    
    if(pmtr.PV_Energy.SPLIT[0]==0xFF && pmtr.PV_Energy.SPLIT[1]==0xFF)
    {
        pmtr.PV_Energy.SPLIT_parameter=0;
        PV_Energy = 0;
    }
    else
    {
        PV_Energy = (float)pmtr.PV_Energy.SPLIT_parameter * 0.1;
    }
    init_timer0();

    while(1)
    {
        /* TODO <INSERT USER APPLICATION CODE HERE> */

        if(function.interruptbit==1)
        {
            Average_Parameters(&ADCBUF);
            function.interruptbit=0;
            pointADC = ADCBUF;

            ADCON0bits.CHS = 0; // AN0 channel
            ADCON0bits.GO_DONE = 1; // conversion started
            while(ADCON0bits.ADON == 1);
            AnalogData = (unsigned int)(ADRESH&0x03);
            AnalogData = AnalogData<<8;
            *pointADC++ = AnalogData + (unsigned int)ADRESL;

            ADCON0bits.CHS = 1; // AN1 channel
            ADCON0bits.GO_DONE = 1; // conversion started
            while(ADCON0bits.ADON == 1);
            AnalogData = (unsigned int)(ADRESH&0x03);
            AnalogData = AnalogData<<8;
            *pointADC++ = AnalogData + (unsigned int)ADRESL;

            ADCON0bits.CHS = 2; // AN2 channel
            ADCON0bits.GO_DONE = 1; // conversion started
            while(ADCON0bits.ADON == 1);
            AnalogData = (unsigned int)(ADRESH&0x03);
            AnalogData = AnalogData<<8;
            *pointADC++ = AnalogData + (unsigned int)ADRESL;

            ADCON0bits.CHS = 3; // AN3 channel
            ADCON0bits.GO_DONE = 1; // conversion started
            while(ADCON0bits.ADON == 1);
            AnalogData = (unsigned int)(ADRESH&0x03);
            AnalogData = AnalogData<<8;
            *pointADC = AnalogData + (unsigned int)ADRESL;

            Average_Parameters(&ADCBUF);
            
            if(parameter.PV_voltage.SPLIT_parameter<parameter.BAT_voltage.SPLIT_parameter)
            {
                PV_presence--;
                if(PV_presence<=0)
                {
                    PV_presence=0;
                    pv.off=1;
                    pv.on=0;
                }
            }
            else
            {

                PV_presence++;
                if(PV_presence>=200)
                {
                    PV_presence=200; // check the state for 50 ms
                    pv.on=1;
                    pv.off=0;
                }
            }
            /* power consumed from PV for charging the battery,assumed that battery is charging when PV is on...
             if pv doesnot charge when the system when the grid is off, then 
             write if(pv.on==1 && grid.on) in place of if(pv.on==1)*/
            if(pv.on==1)
            {
                if(function.changefromPVoff_to_on==0)
                {
                    function.changefromPVoff_to_on=1;
                    BAT_Energy=0; // discarging energy is reset when the day starts...
                    PV_Energy = Actual_Energy_remains;
                    pmtr.PV_Energy.SPLIT_parameter = (unsigned int)(PV_Energy*10);
                    pv_secondcal = 0;
                    initiate_EEPROMwrite=0;
                }
                function.changefromPVon_to_off=0;
                if(initiate_EEPROMwrite==0)
                {
                    initiate_EEPROMwrite=1;
                }
                if(pv_secondcal>=4000) // 1 sec
                {
                   pv_secondcal=0;
                   PV_power = (float)parameter.PV_current.SPLIT_parameter*PVI_factor*(float)parameter.PV_voltage.SPLIT_parameter*PVV_factor; // PV_power in Watt-hour
                   PV_Energy = PV_Energy + (float)PV_power*0.028*0.01;  // PV_power is calculated in every seconds...
                                                                                       // Energy is 3600 seconds.
                   pmtr.PV_Energy.SPLIT_parameter = (unsigned int)PV_Energy*10;
                }
               else
                {
                   pv_secondcal++;
                }
            }
            else if(grid.off==1) //condition for discharging of battery,system will run on inverter...
            {
                
                if(batt_secondcal>4000) // 1 sec
                 {
                   batt_secondcal=0;
                   /* When PV is not there,it will calculate the Batterry discharge current and subtract it from PV */
                   BAT_power = (float)parameter.BAT_voltage.SPLIT_parameter*BTV_factor*(float)parameter.BAT_current.SPLIT_parameter*BTI_factor; // battery_power in Watt-hour
                   BAT_Energy = BAT_Energy + (float)BAT_power*0.028*0.01;  // PV_power is calculated in every seconds...
                                                                                      // Energy is 3600 seconds.
                   pmtr.BT_Energy.SPLIT_parameter = (unsigned int)BAT_Energy*10;
                  }
                else
                 {
                   batt_secondcal++;
                  }
                Actual_Energy_remains = PV_Energy-BAT_Energy;
                if(Actual_Energy_remains<=0)
                {
                   Actual_Energy_remains=0;
                 }
               
            }
            if(pv.off==1)
            {
                if(function.changefromPVon_to_off==0)
                {
                    function.changefromPVon_to_off=1;
                    BAT_Energy=0; // discarging energy is reset when the day starts...
                    PV_Energy = Actual_Energy_remains;
                    pv_secondcal = 0;
                    initiate_EEPROMwrite=0;
                }
                function.changefromPVoff_to_on=0;
                if(initiate_EEPROMwrite==0)
                {
                    initiate_EEPROMwrite=1;
                }
            }
           if(PORTCbits.RC0==1) // High means Mains is present
            {
               grid_check++;
               if(grid_check>=40) //check for 10 ms
               {
                  grid.on=1;
                  grid.off=0;
                  grid_check=40;
               }
            }
           else
            {
               grid_check--;
               if(grid_check<=0)
               {
                  grid.off=1;
                  grid.on=0;
                  grid_check=0;
               }
            }

            /*For checking the status of the switch to reset the energy*/
            if(PORTCbits.RC1==1) // switch status
            {
               switch_status++;
               if(switch_status>=14400000) //check for 1 second
               {
                  press_sw.on=1;
                  press_sw.off=0;
                  switch_status=14400000;
               }
            }
           else
            {
               switch_status--;
               if(switch_status<=0)
               {
                  press_sw.off=1;
                  press_sw.on=0;
                  switch_status=0;
               }
            }
        }
        if(press_sw.on==1)
        {
              // erase the PV Energy value and store...
               write_EEPROM(0xFF ,0x00 );
               write_EEPROM(0xFF ,0x01 );
               pmtr.PV_Energy.SPLIT_parameter=0;
        }
        if(initiate_EEPROMwrite==1)
        {
               // write the PV Energy value to be stored...
               write_EEPROM(pmtr.PV_Energy.SPLIT[1] ,0x00 );  // Higher byte stored in 0x00
               write_EEPROM(pmtr.PV_Energy.SPLIT[0] ,0x01 );  // Lower byte stored in 0x01
               initiate_EEPROMwrite=2;
        }
        if(initiate_EEPROMwrite==2)
        {

        }
        /* If battery discharging load > 250 wh, then mains will be bypassed...*/
        if(BAT_Energy>2500 || Actual_Energy_remains==0)
        {
             RELAY_ON
        }
        else
        {
             RELAY_OFF
        }
    }

}
/*global interrupt ISR*/
void interrupt isr(void)
{

/* isr code goes here */
   if(INTCONbits.T0IF == 1)   // Timer 0 interrupt enable
   {
       function.interruptbit = 1;
       INTCONbits.T0IF = 0;
       TMR0 = 131;
       
   }
}

